package com.bocloud.cmp.provider;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.common.enums.TceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.common.TceLocationProvider;
import com.tencentcloudapi.billing.v20180709.BillingClient;
import com.tencentcloudapi.bm.v20180423.BmClient;
import com.tencentcloudapi.cbs.v20170312.CbsClient;
import com.tencentcloudapi.clb.v20180317.ClbClient;
import com.tencentcloudapi.common.AbstractClient;
import com.tencentcloudapi.common.Credential;
import com.tencentcloudapi.common.profile.ClientProfile;
import com.tencentcloudapi.common.profile.HttpProfile;
import com.tencentcloudapi.cvm.v20170312.CvmClient;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesResponse;
import com.tencentcloudapi.vpc.v20170312.VpcClient;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: Tceprovider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
public class TceProvider extends GenericProvider implements SuperProvider {

	private static Logger logger = LoggerFactory.getLogger(TceProvider.class);

	private String secretId;
	private String secretKey;
	private Credential credential;
	private String endpoint; // 平台地址 m14-2.tcecqpoc.fsphere.cn
	private Butler butler;
	private String projectId = "0"; // 默认项目
	private String regionId = "nanjing"; // 选择大区
	private ClientProfile clientProfile = new ClientProfile(); // 接入地域
	private HttpProfile httpProfile = new HttpProfile();
	private static final String HTTP = "http://";
	private static final Integer CONN_TIMEOUT = 60 * 10;
	private static final String APIVERSION = "api3";
	protected static final Integer LIMIT = 50;

	/**
	 * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
	 *
	 * @param butler
	 *            云平台认证信息
	 */
	public TceProvider(Butler butler) {
		super(butler);
		this.secretId = butler.getAccessKey();
		this.secretKey = butler.getSecretKey();
		this.regionId = butler.getRegionId();
		this.credential = new Credential(secretId, secretKey);
		this.endpoint = APIVERSION + '.' + butler.getUrl();
		this.butler = butler;
	}

	private void configuration(TceEnum type) {
		httpProfile.setConnTimeout(CONN_TIMEOUT);
		httpProfile.setProtocol(HTTP);
		httpProfile.setEndpoint(type.getValue() + '.' + endpoint);
		clientProfile.setHttpProfile(httpProfile);
	}

	public AbstractClient getClient(TceEnum type) {
		AbstractClient abstractClient = null;
		this.configuration(type);
		switch (type) {
		case CVM:
			abstractClient = new CvmClient(credential, regionId, clientProfile);
			break;
		case CBS:
			abstractClient = new CbsClient(credential, regionId, clientProfile);
			break;
		case VPC:
			abstractClient = new VpcClient(credential, regionId, clientProfile);
			break;
		case CLB:
			abstractClient = new ClbClient(credential, regionId, clientProfile);
			break;
		case BM:
			abstractClient = new BmClient(credential, regionId, clientProfile);
			break;
		case BILL:
			abstractClient = new BillingClient(credential, regionId, clientProfile);
			break;
		default:
			break;
		}
		return abstractClient;
	}

	public static void main(String[] args) {
		Butler butler = new Butler();
		butler.setUrl("yf-1.tcepoc.fsphere.cn");
		butler.setSecretKey("mFPItB8l58vPr34tPdaEZcNpySdW7PO5");
		butler.setAccessKey("AKIDWQ7ZmofLp7B3kGZgbY5PSDelZ6MJwKRQ");
		butler.setRegionId("chongqing");
		TceLocationProvider provider = new TceLocationProvider(butler);
		System.out.println(provider.sync());
		// TceProvider provider = new TceProvider(butler);
		// BillingClient client = (BillingClient) provider.getClient(TceEnum.BILL);
		// Long offset = 0L;
		// DescribeBillDetailRequest req = new DescribeBillDetailRequest();
		// List<BillDetail> billDetails = new ArrayList<>();
		// try {
		// req.setOffset(offset);
		// req.setBeginTime("2020-01-03 00:00:00");
		// req.setEndTime("2020-07-03 00:00:00");
		// req.setPeriodType("byPayTime");
		// req.setLimit(50L);
		// DescribeBillDetailResponse response = client.DescribeBillDetail(req);
		// System.out.println(response.getRequestId());
		// billDetails.addAll(Arrays.asList(response.getDetailSet()));
		// // 循环分页查询
		// while (billDetails.size() >= LIMIT + offset) {
		// req.setOffset(offset += LIMIT);
		// response = client.DescribeBillDetail(req);
		// billDetails.addAll(Arrays.asList(response.getDetailSet()));
		// }
		// System.out.println(JSONObject.toJSON(billDetails));
		// } catch (Exception e) {
		// System.out.println(e.getMessage());
		// }
	}

	@Override
	public boolean accessable() {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			DescribeInstancesRequest req = new DescribeInstancesRequest();
			DescribeInstancesResponse response = client.DescribeInstances(req);
			response.getInstanceSet();
			return true;
		} catch (Exception e) {
			logger.error("云平台(SDK)认证失败 ：" + e.getMessage());
			return false;
		}
	}

	public String getSecretId() {
		return secretId;
	}

	public void setSecretId(String secretId) {
		this.secretId = secretId;
	}

	public String getSecretKey() {
		return secretKey;
	}

	public void setSecretKey(String secretKey) {
		this.secretKey = secretKey;
	}

	public Butler getButler() {
		return butler;
	}

	public void setButler(Butler butler) {
		this.butler = butler;
	}

	public String getRegionId() {
		return regionId;
	}

	public void setRegionId(String regionId) {
		this.regionId = regionId;
	}

	public String getProjectId() {
		return projectId;
	}

	public void setProjectId(String projectId) {
		this.projectId = projectId;
	}
}
