package com.bocloud.cmp.provider;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.TceActionEnum;
import com.bocloud.cmp.common.enums.TceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.MapTools;
import org.apache.commons.codec.binary.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.TreeMap;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: TceRestApiProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
public class TceRestApiProvider extends GenericProvider implements SuperProvider {

	private static Logger logger = LoggerFactory.getLogger(TceRestApiProvider.class);

	private String secretId;
	private String secretKey;
	private Butler butler;
	private String domain;
	private String endpoint; // 平台地址
								// api2.m14-2.tcecqpoc.fsphere.cn/v2/index.php?
	private String regionId = "nanjing";
	private String projectId = "0"; // 默认项目
	private static final String CHARSET = "UTF-8";
	private static final String HTTP = "http://";
	private static final String GET = "GET";
	private static final String APIVERSION = "api2";
	private static final String APISUFFIX = "/v2/index.php?";
	protected static final String MESSAGE = "message";
	protected static final Integer LIMIT = 50; // 腾讯资源查询时用来控制分页的参数，v3最大为100，v2最大为50
	protected static Integer subscript = 0; // 腾讯个别创建资源需要下标

	public TceRestApiProvider(Butler butler) {
		super(butler);
		this.secretId = butler.getAccessKey();
		this.secretKey = butler.getSecretKey();
		this.regionId = butler.getRegionId();
		this.endpoint = APIVERSION + '.' + butler.getUrl() + APISUFFIX;
		this.butler = butler;
	}

	private void configuration(TceEnum type) {
		this.domain = type.getValue() + '.' + endpoint;
	}

	public boolean verify(JSONObject object) {
		if (null == object || object.isEmpty()) {
			return false;
		}
		return object.getInteger("code") == 0 && object.getString("codeDesc").equals("Success");
	}

	@Override
	public boolean accessable() {
		TreeMap<String, Object> params = new TreeMap<>();
		JSONObject object = this.connection(TceEnum.VPC, TceActionEnum.LIST_VPC, params);
		return this.verify(object);
	}

	public JSONObject connection(TceEnum type, TceActionEnum action, TreeMap<String, Object> param) {
		this.configuration(type);// 获取了连接域名
		return this.invoke(action.getValue(), this.domain, this.secretId, this.secretKey, this.regionId, HTTP, param);
	}

	public JSONObject invoke(String action, String domain, String secretId, String secretKey, String region,
			String http, TreeMap<String, Object> param) {
		try {
			TreeMap<String, Object> params = new TreeMap<>();
			params.put("Action", action); // 公共参数
			params.put("Nonce", (int) ((Math.random() * 9 + 1) * 10000)); // 公共参数
			params.put("Region", region); // 公共参数
			params.put("SecretId", secretId); // 公共参数
			params.put("Timestamp", System.currentTimeMillis() / 1000);
			params.putAll(param);
			String srcStr = getStringToSign(params, domain);
			params.put("Signature", genHMAC(srcStr, secretKey));
			String url = getUrl(params, http + domain);
			HttpClient httpClient = new HttpClient();
			return JSONTools.isJSONObj(httpClient.get(url).getMessage());
		} catch (Exception e) {
			logger.error("云平台API连接失败： " + e.getMessage());
			return new JSONObject(MapTools.simpleMap(MESSAGE, e.getMessage()));
		}
	}

	// 公共方法-获取地址 通用
	public String getUrl(TreeMap<String, Object> params, String domain) throws UnsupportedEncodingException {
		StringBuilder url = new StringBuilder(domain);
		// 实际请求的url中对参数顺序没有要求
		for (String k : params.keySet()) {
			// 需要对请求串进行urlencode，由于key都是英文字母，故此处仅对其value进行urlencode
			url.append(k).append("=").append(URLEncoder.encode(params.get(k).toString(), CHARSET)).append("&");
		}
		return url.toString().substring(0, url.length() - 1);
	}

	// 公共方法-字典排序 通用
	public String getStringToSign(TreeMap<String, Object> params, String domain) {
		StringBuilder s2s = new StringBuilder(GET + domain);
		// 签名时要求对参数进行字典排序，此处用TreeMap保证顺序
		for (String k : params.keySet()) {
			s2s.append(k).append("=").append(params.get(k).toString()).append("&");
		}
		return s2s.toString().substring(0, s2s.length() - 1);
	}

	public String genHMAC(String srcStr, String secretKey) throws Exception {
		byte[] result = null;
		// 根据给定的字节数组构造一个密钥,第二参数指定一个密钥算法的名称
		SecretKeySpec signinKey = new SecretKeySpec(secretKey.getBytes(), "HmacSHA1");
		// 生成一个指定 Mac 算法 的 Mac 对象
		Mac mac = Mac.getInstance("HmacSHA1");
		// 用给定密钥初始化 Mac 对象
		mac.init(signinKey);
		// 完成 Mac 操作
		byte[] rawHmac = mac.doFinal(srcStr.getBytes());
		result = Base64.encodeBase64(rawHmac);
		if (null != result) {
			return new String(result);
		} else {
			return null;
		}
	}

	public String getProjectId() {
		return projectId;
	}

	public void setProjectId(String projectId) {
		this.projectId = projectId;
	}

	public String getSecretId() {
		return secretId;
	}

	public void setSecretId(String secretId) {
		this.secretId = secretId;
	}

	public String getSecretKey() {
		return secretKey;
	}

	public void setSecretKey(String secretKey) {
		this.secretKey = secretKey;
	}

	public Butler getButler() {
		return butler;
	}

	public void setButler(Butler butler) {
		this.butler = butler;
	}

	public String getDomain() {
		return domain;
	}

	public void setDomain(String domain) {
		this.domain = domain;
	}

	public String getRegionId() {
		return regionId;
	}

	public void setRegionId(String regionId) {
		this.regionId = regionId;
	}
}
