package com.bocloud.cmp.provider.common;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.common.enums.TceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.TceProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tceconvertor.FlavorConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.cvm.v20170312.CvmClient;
import com.tencentcloudapi.cvm.v20170312.models.DescribeZoneInstanceConfigInfosRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeZoneInstanceConfigInfosResponse;
import com.tencentcloudapi.cvm.v20170312.models.Filter;
import com.tencentcloudapi.cvm.v20170312.models.InstanceTypeConfig;
import com.tencentcloudapi.cvm.v20170312.models.InstanceTypeQuotaItem;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: TceFlavorProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.tce")
public class TceFlavorProvider extends TceProvider implements FlavorProvider {

	private static Logger logger = LoggerFactory.getLogger(TceFlavorProvider.class);

	private BeanConvertor<FlavorModel, InstanceTypeConfig> convertor = new FlavorConvertor();

	public TceFlavorProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult sync() {
		BocloudResult result;
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			DescribeZoneInstanceConfigInfosRequest req = new DescribeZoneInstanceConfigInfosRequest();
			Filter f1 = new Filter();
			f1.setName("instance-charge-type");
			// PREPAID：表示预付费，即包年包月 | POSTPAID_BY_HOUR：表示后付费，即按量计费
			f1.setValues(new String[] { "POSTPAID_BY_HOUR" });
			Filter[] filters = new Filter[] { f1 };
			req.setFilters(filters);
			DescribeZoneInstanceConfigInfosResponse resp = client.DescribeZoneInstanceConfigInfos(req);
			result = new BocloudResult(true, convertList(Arrays.asList(resp.getInstanceTypeQuotaSet())),
					"list flavor success!");
		} catch (Exception e) {
			logger.error("list flavor error :", e.getMessage());
			result = new BocloudResult(false, e.getMessage());
		}
		return result;
	}

	public List<FlavorModel> convertList(List<? extends InstanceTypeQuotaItem> objects) {
		List<FlavorModel> regionModels = new ArrayList<>();
		for (InstanceTypeQuotaItem object : objects) {
			String zone = object.getZone();
			if (StringUtils.isEmpty(zone)) {
				continue;
			}
			String status = object.getStatus();
			if ("SELL".equalsIgnoreCase(status)) {
				regionModels.add(convertModel(object));
			}
		}
		return regionModels;
	}

	public FlavorModel convertModel(InstanceTypeQuotaItem info) {
		FlavorModel model = new FlavorModel();
		if (null != info) {
			model.setCpu(info.getCpu());
			model.setMemory(BigDecimal.valueOf(info.getMemory()));
			model.setName(info.getTypeName());
			model.setFamily(info.getInstanceFamily());
			model.setFlavorUuid(info.getInstanceType());
			model.setDisk(info.getNetworkCard());
			model.setInstanceChargeType(info.getInstanceChargeType());
			model.setStatus(info.getStatus());
			model.setZone(info.getZone());
		}
		return model;
	}

	@Override
	public BocloudResult create(FlavorModel flavor) {
		return null;
	}

	@Override
	public BocloudResult remove(String flavorId) {
		return null;
	}

	@Override
	public BocloudResult update(FlavorModel flavorModel) {
		return null;
	}

}
