package com.bocloud.cmp.provider.compute;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.TreeMap;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.TceActionEnum;
import com.bocloud.cmp.common.enums.TceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.OSMigrateModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.TceProvider;
import com.bocloud.cmp.provider.TceRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.storage.TceVolumeProvider;
import com.bocloud.cmp.tceconvertor.ServerConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.cvm.v20170312.CvmClient;
import com.tencentcloudapi.cvm.v20170312.models.AssociateInstancesKeyPairsRequest;
import com.tencentcloudapi.cvm.v20170312.models.DataDisk;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesResponse;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesStatusRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesStatusResponse;
import com.tencentcloudapi.cvm.v20170312.models.DisassociateInstancesKeyPairsRequest;
import com.tencentcloudapi.cvm.v20170312.models.Instance;
import com.tencentcloudapi.cvm.v20170312.models.InstanceChargePrepaid;
import com.tencentcloudapi.cvm.v20170312.models.InstanceStatus;
import com.tencentcloudapi.cvm.v20170312.models.InternetAccessible;
import com.tencentcloudapi.cvm.v20170312.models.LoginSettings;
import com.tencentcloudapi.cvm.v20170312.models.ModifyInstancesAttributeRequest;
import com.tencentcloudapi.cvm.v20170312.models.Placement;
import com.tencentcloudapi.cvm.v20170312.models.RebootInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.ResetInstanceRequest;
import com.tencentcloudapi.cvm.v20170312.models.ResetInstancesInternetMaxBandwidthRequest;
import com.tencentcloudapi.cvm.v20170312.models.ResetInstancesTypeRequest;
import com.tencentcloudapi.cvm.v20170312.models.ResizeInstanceDisksRequest;
import com.tencentcloudapi.cvm.v20170312.models.RunInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.RunInstancesResponse;
import com.tencentcloudapi.cvm.v20170312.models.StartInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.StopInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.SystemDisk;
import com.tencentcloudapi.cvm.v20170312.models.TerminateInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.VirtualPrivateCloud;
import com.tencentcloudapi.vpc.v20170312.VpcClient;
import com.tencentcloudapi.vpc.v20170312.models.AssociateAddressRequest;
import com.tencentcloudapi.vpc.v20170312.models.DisassociateAddressRequest;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: TceServerProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.tce")
public class TceServerProvider extends TceProvider implements ServerProvider {

	private static Logger logger = LoggerFactory.getLogger(TceServerProvider.class);

	private BeanConvertor<ServerModel, Instance> convertor = new ServerConvertor();

	private Butler butler;

	public TceServerProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult list() {
		Integer offset = 0;
		DescribeInstancesRequest req = new DescribeInstancesRequest();
		List<Instance> instances = new ArrayList<>();
		try {
			req.setLimit(LIMIT);
			DescribeInstancesResponse response = ((CvmClient) this.getClient(TceEnum.CVM)).DescribeInstances(req);
			instances.addAll(Arrays.asList(response.getInstanceSet()));
			// 循环分页查询
			while (instances.size() >= LIMIT + offset) {
				req.setOffset(offset += LIMIT);
				response = ((CvmClient) this.getClient(TceEnum.CVM)).DescribeInstances(req);
				instances.addAll(Arrays.asList(response.getInstanceSet()));
			}
			List<ServerModel> models = convertor.convertList(instances);
			Optional.ofNullable(models).ifPresent(
					serverModels -> serverModels.stream().forEach(
							serverModel -> serverModel.setRegionId(this.getRegionId())));
			return new BocloudResult(true, models, "list cloudServer success!");
		} catch (Exception e) {
			logger.error("list cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	public BocloudResult getInstances(String instanceId) {
		BocloudResult result = new BocloudResult(false, "data does not exist");
		try {
			Long startTime = System.currentTimeMillis();
			Long endTime = System.currentTimeMillis();
			// 轮询5m
			while (endTime - startTime < 5 * 60 * 1000) {
				// 主机创建时间不确定，增加休眠减少轮询次数
				Thread.sleep(6000);
				BocloudResult detail = this.detail(instanceId);
				ServerModel model = JSONObject
						.parseObject(JSONObject.toJSONString(detail.getData()), ServerModel.class);
				if (ObjectUtils.isEmpty(model)) {
					break;
				}
				if ("RUNNING".equalsIgnoreCase(model.getStatus()) || "STOPED".equalsIgnoreCase(model.getStatus())
						|| "STOPPED".equalsIgnoreCase(model.getStatus())) {
					result = this.detail(instanceId);
					break;
				}
				endTime = System.currentTimeMillis();
			}
		} catch (Exception e) {
			logger.error("create server get status error： " + e.getMessage());
			result = new BocloudResult(false, "tce server get status error： " + e.getMessage());
		}
		return result;
	}

	// 私有云5月份版本查询主机详情没有主机状态，12月份查询后有该状态，暂时还是以5月版本为主
	public BocloudResult getStatusInstances(String instanceId) {
		BocloudResult result = new BocloudResult(false, "server data does not exist");
		try {
			Long startTime = System.currentTimeMillis();
			Long endTime = System.currentTimeMillis();
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			DescribeInstancesStatusRequest request = new DescribeInstancesStatusRequest();
			request.setInstanceIds(new String[] { instanceId });
			while (endTime - startTime < 5 * 60 * 1000) {
				Thread.sleep(6000);
				DescribeInstancesStatusResponse resp = client.DescribeInstancesStatus(request);
				InstanceStatus[] instanceStatusSet = resp.getInstanceStatusSet();
				if (instanceStatusSet == null || instanceStatusSet.length <= 0) {
					break;
				}
				InstanceStatus instanceStatus = instanceStatusSet[0];
				if ("RUNNING".equalsIgnoreCase(instanceStatus.getInstanceState())
						|| "STOPED".equalsIgnoreCase(instanceStatus.getInstanceState())
						|| "STOPPED".equalsIgnoreCase(instanceStatus.getInstanceState())) {
					result = this.getStatusDetail(instanceId, instanceStatus.getInstanceState());
					break;
				}
				endTime = System.currentTimeMillis();
			}
		} catch (Exception e) {
			logger.error("server status get status error： " + e.getMessage());
			result = new BocloudResult(false, "server status get status error： " + e.getMessage());
		}
		return result;
	}

	@Override
	public BocloudResult create(ServerModel server) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			if (!StringUtils.isEmpty(server.getRegionId())) {
				client.setRegion(server.getRegionId());
			}
			RunInstancesRequest req = new RunInstancesRequest();
			// 付费方式 PREPAID：预付费，即包年包月 POSTPAID_BY_HOUR：按小时后付费
			String instanceChargeType = server.getInstanceChargeType();
			if (StringUtils.isEmpty(instanceChargeType)) {
				instanceChargeType = "POSTPAID_BY_HOUR";
			}
			req.setInstanceChargeType(instanceChargeType);
			// 购买时长
			if ("PREPAID".equals(instanceChargeType)) {
				InstanceChargePrepaid instanceChargePrepaid = new InstanceChargePrepaid();
				instanceChargePrepaid.setPeriod(Integer.valueOf(server.getInstancePeriod()));
				/**
				 * •NOTIFY_AND_AUTO_RENEW：通知过期且自动续费
				 * •NOTIFY_AND_MANUAL_RENEW：通知过期不自动续费
				 * •DISABLE_NOTIFY_AND_MANUAL_RENEW：不通知过期不自动续费
				 * 默认取值：NOTIFY_AND_MANUAL_RENEW
				 * 。若该参数指定为NOTIFY_AND_AUTO_RENEW，在账户余额充足的情况下，实例到期后将按月自动续费。
				 */
				if (!StringUtils.isEmpty(server.getInstanceRenewFlag())) {
					instanceChargePrepaid.setRenewFlag(server.getInstanceRenewFlag());
				}
				req.setInstanceChargePrepaid(instanceChargePrepaid);
			}
			// 地区 项目
			Placement placement = new Placement();
			placement.setZone(server.getZone());
			if (!StringUtils.isEmpty(this.getProjectId())) {
				placement.setProjectId(Integer.valueOf(this.getProjectId()));
			}
			req.setPlacement(placement);
			req.setImageId(server.getImageUuid());
			if (!StringUtils.isEmpty(server.getName())) {
				req.setInstanceName(server.getName());
			}
			if (!StringUtils.isEmpty(server.getFlavorUuid())) {
				req.setInstanceType(server.getFlavorUuid()); // 规格
			}
			// LOCAL_BASIC：本地硬盘 || LOCAL_SSD：本地SSD硬盘 || CLOUD_BASIC：普通云硬盘 ||
			// CLOUD_SSD：SSD云硬盘 || CLOUD_PREMIUM：高性能云硬盘
			SystemDisk systemDisk = new SystemDisk();
			systemDisk.setDiskSize(Integer.valueOf(server.getSystemDiskSize()));
			// 默认SSD云硬盘
			if (!StringUtils.isEmpty(server.getSystemDiskCategory())) {
				systemDisk.setDiskType(server.getSystemDiskCategory());
			}
			req.setSystemDisk(systemDisk);
			// volume
			List<VolumeModel> diskDevices = server.getDiskDevices();
			if (!CollectionUtils.isEmpty(diskDevices)) {
				DataDisk disk = null;
				List<DataDisk> diskList = new ArrayList<>();
				for (VolumeModel volumeModel : diskDevices) {
					disk = new DataDisk();
					disk.setDiskSize(Integer.valueOf(volumeModel.getSize()));
					diskList.add(disk);
				}
				DataDisk[] dataDisks = new DataDisk[diskList.size()];
				req.setDataDisks(diskList.toArray(dataDisks));
			}
			// vpc
			String subnetUuid = server.getSubnetUuid();
			String networkUuid = server.getNetworkUuid();
			if (null != subnetUuid && null != networkUuid) {
				VirtualPrivateCloud virtualPrivateCloud = new VirtualPrivateCloud();
				virtualPrivateCloud.setSubnetId(server.getSubnetUuid());
				virtualPrivateCloud.setVpcId(server.getNetworkUuid());
				virtualPrivateCloud.setAsVpcGateway(true);
				req.setVirtualPrivateCloud(virtualPrivateCloud);
			}
			Integer count = server.getCount();
			if (null == count || count <= 0) {
				count = 1;
			}
			req.setInstanceCount(Integer.valueOf(count));
			// 登录方式
			LoginSettings loginSettings = new LoginSettings();
			String password = server.getPassword();
			if (!StringUtils.isEmpty(password)) {
				loginSettings.setPassword(password);
			}
			String keypairId = server.getKeypairName();
			if (!StringUtils.isEmpty(keypairId)) {
				loginSettings.setKeyIds(new String[] { keypairId });
			}
			req.setLoginSettings(loginSettings);

			List<String> securityGroups = server.getSecurityGroups();
			if (!CollectionUtils.isEmpty(securityGroups)) {
				String[] securityGroupIds = new String[securityGroups.size()];
				req.setSecurityGroupIds(securityGroups.toArray(securityGroupIds));
			}
			RunInstancesResponse resp = client.RunInstances(req);
			String[] instanceIdSet = resp.getInstanceIdSet();
			return getStatusInstances(instanceIdSet[0]);
		} catch (Exception e) {
			logger.error("create cloudServer error:{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(ServerModel server) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			ModifyInstancesAttributeRequest req = new ModifyInstancesAttributeRequest();
			req.setInstanceIds(new String[] { server.getInstanceId() });
			String name = server.getName();
			if (!StringUtils.isEmpty(name)) {
				req.setInstanceName(name);
			}
			List<String> securityGroups = server.getSecurityGroups();
			if (!CollectionUtils.isEmpty(securityGroups)) {
				String[] groups = new String[securityGroups.size()];
				req.setSecurityGroups(securityGroups.toArray(groups));
			}
			client.ModifyInstancesAttribute(req);
			BocloudResult detail = this.detail(server.getInstanceId());
			return new BocloudResult(true, detail.getData(), "修改成功");
		} catch (Exception e) {
			logger.error("modify cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult migrate(ServerModel server) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult migrate(OSMigrateModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult remove(String instanceId, String name) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			TerminateInstancesRequest request = new TerminateInstancesRequest();
			request.setInstanceIds(new String[] { instanceId });
			client.TerminateInstances(request);
			return new BocloudResult(true, "remove cloudServer success!");
		} catch (Exception e) {
			logger.error("remove cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String instanceId) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			DescribeInstancesRequest req = new DescribeInstancesRequest();
			req.setInstanceIds(new String[] { instanceId });
			DescribeInstancesResponse resp = client.DescribeInstances(req);
			Instance[] instanceSet = resp.getInstanceSet();
			ServerModel convertModel = null;
			if (null != instanceSet && instanceSet.length > 0) {
				Instance instance = instanceSet[0];
				convertModel = convertor.convertModel(instance);
				convertModel.setRegionId(this.getRegionId());
				List<VolumeModel> volumeDetail = this.getVolumeDetail(convertModel);
				if (!CollectionUtils.isEmpty(volumeDetail)) {
					convertModel.setDiskDevices(volumeDetail);
				}
			}
			return new BocloudResult(true, convertModel, "detail cloudServer success!");
		} catch (Exception e) {
			logger.error("detail cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	public BocloudResult getStatusDetail(String instanceId, String serverStatus) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			DescribeInstancesRequest req = new DescribeInstancesRequest();
			req.setInstanceIds(new String[] { instanceId });
			DescribeInstancesResponse resp = client.DescribeInstances(req);
			Instance[] instanceSet = resp.getInstanceSet();
			ServerModel convertModel = null;
			if (null != instanceSet && instanceSet.length > 0) {
				Instance instance = instanceSet[0];
				convertModel = convertor.convertModel(instance);
				convertModel.setRegionId(this.getRegionId());
				if (!StringUtils.isEmpty(serverStatus)) {
					convertModel.setStatus(serverStatus);
				}
				List<VolumeModel> volumeDetail = this.getVolumeDetail(convertModel);
				if (null != volumeDetail) {
					convertModel.setDiskDevices(volumeDetail);
				}
			}
			return new BocloudResult(true, convertModel, "detail cloudServer success!");
		} catch (Exception e) {
			logger.error("detail status cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, "detail status cloudServer error :{}" + e.getMessage());
		}
	}

	private List<VolumeModel> getVolumeDetail(ServerModel convertModel) {
		List<VolumeModel> result = null;
		try {
			TceVolumeProvider volume = new TceVolumeProvider(butler);
			Integer count = 3;
			while (count > 0) {
				BocloudResult list = volume.list(convertModel.getInstanceId());
				List<VolumeModel> parseArray = JSONObject.parseArray(JSON.toJSONString(list.getData()),
						VolumeModel.class);
				if (parseArray.size() > 0) {
					result = parseArray;
					count = -1;
				} else {
					count--;
					Thread.sleep(2000);
				}
			}
		} catch (Exception e) {
			logger.error("detail volumeStatus error :{}", e.getMessage());
		}
		return result;
	}

	@Override
	public BocloudResult start(String instanceId) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			StartInstancesRequest req = new StartInstancesRequest();
			req.setInstanceIds(new String[] { instanceId });
			client.StartInstances(req);
			return new BocloudResult(true, "start cloudServer success!");
		} catch (Exception e) {
			logger.error("start cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult stop(String instanceId) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			StopInstancesRequest req = new StopInstancesRequest();
			req.setInstanceIds(new String[] { instanceId });
			client.StopInstances(req);
			return new BocloudResult(true, "stop cloudServer success!");
		} catch (Exception e) {
			logger.error("stop cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult reboot(String instanceId) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			RebootInstancesRequest req = new RebootInstancesRequest();
			req.setInstanceIds(new String[] { instanceId });
			client.RebootInstances(req);
			return new BocloudResult(true, "reboot cloudServer success!");
		} catch (Exception e) {
			logger.error("reboot cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult softReboot(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult pause(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult unPause(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult suspend(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult resume(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult joinSGroup(String instanceId, String group) {
		try {
			// 查询当前实例的安全组
			BocloudResult bocloudResult = this.detail(instanceId);
			if (bocloudResult.isFailed()) {
				return new BocloudResult(false, bocloudResult.getMessage(), "修改安全组失败");
			}
			ServerModel modes = (ServerModel) bocloudResult.getData();
			List<String> securityGroups = new ArrayList<>(modes.getSecurityGroups());
			// 加入安全组
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			ModifyInstancesAttributeRequest request = new ModifyInstancesAttributeRequest();
			request.setInstanceIds(new String[] { instanceId });
			securityGroups.add(group);
			String[] strings = new String[securityGroups.size()];
			request.setSecurityGroups(securityGroups.toArray(strings));
			client.ModifyInstancesAttribute(request);
			return new BocloudResult(true, "joinSGroup cloudServer success!");
		} catch (Exception e) {
			logger.error("joinSGroup cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult leaveSGroup(String instanceId, String group) {
		try {
			// 查询当前实例的安全组
			BocloudResult bocloudResult = this.detail(instanceId);
			if (bocloudResult.isFailed()) {
				return new BocloudResult(false, bocloudResult.getMessage(), "当前实例不存在");
			}
			ServerModel modes = (ServerModel) bocloudResult.getData();
			List<String> asList = new ArrayList<>(modes.getSecurityGroups());
			// 移除安全组
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			ModifyInstancesAttributeRequest request = new ModifyInstancesAttributeRequest();
			request.setInstanceIds(new String[] { instanceId });
			asList.remove(group);
			String[] strings = new String[asList.size()];
			request.setSecurityGroups(asList.toArray(strings));
			client.ModifyInstancesAttribute(request);
			return new BocloudResult(true, "leaveSGroup cloudServer success!");
		} catch (Exception e) {
			logger.error("leaveSGroup cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		return null;
	}

	@Override
	public BocloudResult vncConsole(String vncType, String serverId) {
		return null;
	}

	@Override
	public BocloudResult changePassword(String instanceId, String password) {
		try {
			TreeMap<String, Object> params = new TreeMap<>();
			params.put("InstanceIds.0", instanceId);
			params.put("Password", password);
			params.put("ForceStop", true);
			params.put("Version", "2017-03-12");
			TceRestApiProvider tceRestApiProvider = new TceRestApiProvider(butler);
			tceRestApiProvider.connection(TceEnum.CVM, TceActionEnum.ResetInstancesPassword, params);
			Thread.sleep(8000);
			return new BocloudResult(true, "修改密码成功！");
		} catch (Exception e) {
			logger.error("changePassword cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult attachNetwork(ServerModel server) {
		try {
			VpcClient client = (VpcClient) this.getClient(TceEnum.VPC);
			AssociateAddressRequest request = new AssociateAddressRequest();
			request.setInstanceId(server.getInstanceId());
			request.setAddressId(server.getPublicIps());
			client.AssociateAddress(request);
			return new BocloudResult(true, "attachNetwork cloudServer success!");
		} catch (Exception e) {
			logger.error("attachNetwork cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult detachNetwork(String instanceId, String portUuid) {
		try {
			VpcClient client = (VpcClient) this.getClient(TceEnum.VPC);
			DisassociateAddressRequest request = new DisassociateAddressRequest();
			request.setAddressId(portUuid);
			client.DisassociateAddress(request);
			return new BocloudResult(true, "detachNetwork cloudServer success!");
		} catch (Exception e) {
			logger.error("detachNetwork cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId) {
		return null;
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			ResetInstancesTypeRequest req = new ResetInstancesTypeRequest();
			req.setInstanceIds(new String[] { instanceId });
			req.setInstanceType(flavorId);
			req.setForceStop(forceStop);
			client.ResetInstancesType(req);
			BocloudResult result = this.getStatusInstances(instanceId);
			if (result.isFailed()) {
				return result;
			}
			return new BocloudResult(true, result.getData(), "resize cloudServer success!");
		} catch (Exception e) {
			logger.error("resize cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult confirmResize(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult revertResize(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean MemHotEnable) {
		return null;
	}

	@Override
	public BocloudResult getTicket(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult createInResPool(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult mountToolsInstaller(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult unmountToolsInstaller(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult upgradeToolsTask(String instanceId, String options) {
		return null;
	}

	@Override
	public BocloudResult getNetCard(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult getVmIps(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult markAsTemplate(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
		try {
			TreeMap<String, Object> params = new TreeMap<>();
			params.put("InstanceIds.0", instanceId);
			params.put("Password", newPasswd);
			params.put("ForceStop", true);
			params.put("Version", "2017-03-12");
			TceRestApiProvider tceRestApiProvider = new TceRestApiProvider(butler);
			tceRestApiProvider.connection(TceEnum.CVM, TceActionEnum.ResetInstancesPassword, params);
			Thread.sleep(8000);
			return new BocloudResult(true, this.detail(instanceId), "修改密码成功！");
		} catch (Exception e) {
			logger.error("changePassword cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	/**
	 * 虚拟机磁盘扩容
	 */
	@Override
	public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			ResizeInstanceDisksRequest request = new ResizeInstanceDisksRequest();
			request.setInstanceId(instanceId);
			request.setForceStop(false); // 默认不强制关机
			DataDisk dataDisk = new DataDisk();
			dataDisk.setDiskSize(Integer.valueOf(extendSize));
			request.setDataDisks(new DataDisk[] { dataDisk });
			client.ResizeInstanceDisks(request);
			return new BocloudResult(true, "diskExtend cloudServer success!");
		} catch (Exception e) {
			logger.error("diskExtend cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult getMetric(String type) {
		return null;
	}

	@Override
	public BocloudResult vmrcConsole(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult migrate(H3cMigrateModel model) {
		return null;
	}

	@Override
	public BocloudResult list(String hostId) {
		return null;
	}

	@Override
	public BocloudResult create(H3cServerModel arg0) {
		return null;
	}

	@Override
	public BocloudResult getJnlpXml(Long arg0, String arg1) {
		return null;
	}

	@Override
	public BocloudResult resetInstance(ServerModel serverModel) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			ResetInstanceRequest req = new ResetInstanceRequest();
			req.setInstanceId(serverModel.getInstanceId());
			req.setImageId(serverModel.getImageUuid());
			// 登录方式
			LoginSettings loginSettings = new LoginSettings();
			String password = serverModel.getPassword();
			if (!StringUtils.isEmpty(password)) {
				loginSettings.setPassword(password);
			}
			String keypairId = serverModel.getKeypairName();
			if (!StringUtils.isEmpty(keypairId)) {
				loginSettings.setKeyIds(new String[] { keypairId });
			}
			req.setLoginSettings(loginSettings);
			client.ResetInstance(req);
			return new BocloudResult(true, this.getInstancesByImage(serverModel.getInstanceId(),
					serverModel.getImageUuid()).getData(), "resetInstance cloudServer success!");
		} catch (Exception e) {
			logger.error("resetInstance cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	public BocloudResult getInstancesByImage(String instanceId, String imageId) {
		BocloudResult result = new BocloudResult(false, "reset server data does not exist");
		try {
			Long startTime = System.currentTimeMillis();
			Long endTime = System.currentTimeMillis();
			while (endTime - startTime < 5 * 60 * 1000) {
				// 增加休眠减少轮询次数
				Thread.sleep(6000);
				BocloudResult detail = this.detail(instanceId);
				ServerModel serverModel = JSON.parseObject(JSON.toJSONString(detail.getData()), ServerModel.class);
				if (null != serverModel && imageId.equals(serverModel.getImageUuid())) {
					result = this.detail(instanceId);
					break;
				}
				endTime = System.currentTimeMillis();
			}
		} catch (Exception e) {
			logger.error("reset server get status error： " + e.getMessage());
			result = new BocloudResult(false, "reset server get status error： " + e.getMessage());
		}
		return result;
	}

	@Override
	public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			ResetInstancesInternetMaxBandwidthRequest req = new ResetInstancesInternetMaxBandwidthRequest();
			req.setInstanceIds(new String[] { instanceId });
			InternetAccessible internetAccessible = new InternetAccessible();
			internetAccessible.setInternetMaxBandwidthOut(Integer.valueOf(internetMaxBandwidthOut));
			req.setInternetAccessible(internetAccessible);
			client.ResetInstancesInternetMaxBandwidth(req);
			return new BocloudResult(true, "broadband cloudServer success!");
		} catch (Exception e) {
			logger.error("broadband cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult bindKeypair(ServerModel serverModel) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			AssociateInstancesKeyPairsRequest req = new AssociateInstancesKeyPairsRequest();
			req.setInstanceIds(serverModel.getInstanceIds());
			req.setKeyIds(serverModel.getKeypairIds());
			client.AssociateInstancesKeyPairs(req);
			return new BocloudResult(true, "bindKeypair cloudServer succeess!");
		} catch (Exception e) {
			logger.error("bindKeypair cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult unbindKeypair(ServerModel serverModel) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			DisassociateInstancesKeyPairsRequest req = new DisassociateInstancesKeyPairsRequest();
			req.setInstanceIds(serverModel.getInstanceIds());
			req.setKeyIds(serverModel.getKeypairIds());
			client.DisassociateInstancesKeyPairs(req);
			return new BocloudResult(true, "unbindKeypair cloudServer success!");
		} catch (Exception e) {
			logger.error("unbindKeypair cloudServer error :{}", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
		return null;
	}

}