package com.bocloud.cmp.provider.network;

import java.util.TreeMap;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.TceActionEnum;
import com.bocloud.cmp.common.enums.TceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.provider.TceRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.tceconvertor.SecurityGroupConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: TceSecuGroupProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.tce")
public class TceSecuGroupProvider extends TceRestApiProvider implements SecurityGroupProvider {

    private static Logger logger = LoggerFactory.getLogger(TceSecuGroupProvider.class);

    private BeanJsonConvertor<SecurityGroupModel> convertor = new SecurityGroupConvertor();

    public TceSecuGroupProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        TreeMap<String, Object> params = new TreeMap<>();
        JSONObject object = this.connection(TceEnum.DFW, TceActionEnum.LIST_SECURITY_GROUP, params);
        if (this.verify(object)) {
            return new BocloudResult(true, convertor.convertList(object.getJSONArray("data")), "list securityGroup success!");
        } else {
            logger.error("list securityGroup error ： " + object.getString(MESSAGE));
            return new BocloudResult(false, object.getString(MESSAGE));
        }
    }

    @Override
    public BocloudResult create(SecurityGroupModel group) {
        TreeMap<String, Object> params = new TreeMap<>();
        params.put("sgName", group.getName());
        String remark = group.getRemark();
        if (StringUtils.isEmpty(remark)) {
            remark = "";
        }
        params.put("sgRemark", remark);
        JSONObject object = this.connection(TceEnum.DFW, TceActionEnum.CREATE_SECURITY_GROUP, params);
        if (this.verify(object)) {
            JSONObject jsonObject = object.getJSONObject("data");
            // 因为新创建的安全组没有拿到创建时间 所以查询一个detail
            SecurityGroupModel groupModel = convertor
                    .convertModel((JSONObject) this.detail(jsonObject.getString("sgId")).getData());
            groupModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, groupModel, "create securityGroup success!");
        } else {
            logger.error("create securityGroup error ： " + object.getString(MESSAGE));
            return new BocloudResult(false, object.getString(MESSAGE));
        }
    }

    @Override
    public BocloudResult remove(String sgId) {
        TreeMap<String, Object> params = new TreeMap<>();
        params.put("sgId", sgId);
        JSONObject object = this.connection(TceEnum.DFW, TceActionEnum.REMOVE_SECURITY_GROUP, params);
        if (this.verify(object)) {
            return new BocloudResult(true, "remove securityGroup success!");
        } else {
            logger.error("remove securityGroup error ： " + object.getString(MESSAGE));
            return new BocloudResult(false, object.getString(MESSAGE));
        }

    }

    @Override
    public BocloudResult detail(String sgId) {
        TreeMap<String, Object> params = new TreeMap<>();
        params.put("sgId", sgId);
        JSONObject object = this.connection(TceEnum.DFW, TceActionEnum.LIST_SECURITY_GROUP, params);
        if (this.verify(object)) {
            JSONArray jsonArray = object.getJSONArray("data");
            return new BocloudResult(true, jsonArray.get(0), "query securityGroup success!");
        } else {
            logger.error("query securityGroup error ： " + object.getString(MESSAGE));
            return new BocloudResult(false, object.getString(MESSAGE));
        }
    }

    @Override
    public BocloudResult modify(SecurityGroupModel model) {
        return null;
    }
}
