package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.List;
import java.util.TreeMap;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.TceActionEnum;
import com.bocloud.cmp.common.enums.TceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.TceRestApiProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: TceSecuGroupRuleProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.tce")
public class TceSecuGroupRuleProvider extends TceRestApiProvider implements SecurityGroupRuleProvider {

	private static Logger logger = LoggerFactory.getLogger(TceSecuGroupRuleProvider.class);

	public TceSecuGroupRuleProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list(String secuId) {
		TreeMap<String, Object> params = new TreeMap<>();
		params.put("sgId", secuId);
		JSONObject object = this.connection(TceEnum.DFW, TceActionEnum.LIST_SECURITY_GROUP_RULE, params);
		if (this.verify(object)) {
			List<SecurityGroupRuleModel> list = convertModel(object.getJSONObject("data"), secuId);
			return new BocloudResult(true, list, "list securityGroupRule success!");
		} else {
			logger.error("list securityGroupRule error ： " + object.getString(MESSAGE));
			return new BocloudResult(false, object.getString(MESSAGE));
		}
	}

	@Override
	public BocloudResult create(SecurityGroupRuleModel model) {
		TreeMap<String, Object> params = new TreeMap<>();
		params.put("sgId", model.getGroupUuid());
		params.put("direction", model.getDirection());
		params.put("index", model.getSgrIndex());
		String action = model.getAction();
		if (StringUtils.isEmpty(action)) {
			action = "ACCEPT"; // ACCEPT允许 DROPzuzh
		}
		params.put("policys.0.action", action);
		params.put("policys.0.ipProtocol", model.getProtocol());
		params.put("policys.0.cidrIp", model.getRemoteIpPrefix());
		params.put("policys.0.desc", model.getRemark());
		if (!"icmp".equalsIgnoreCase(model.getProtocol())) {
			Integer portMax = model.getPortMax();
			Integer portMin = model.getPortMin();
			if (null == portMax || null == portMin) {
				return new BocloudResult(false, "参数必要参数缺失！");
			}
			if (portMax.equals(portMin)) {
				params.put("policys.0.portRange", portMax + "");
			} else {
				params.put("policys.0.portRange", portMin + "-" + portMax);
			}
		}
		JSONObject object = this.connection(TceEnum.DFW, TceActionEnum.CREATE_SECURITY_GROUP_RULE, params);
		if (this.verify(object)) {
			model.setAction(model.getAction() == null ? "ACCEPT" : model.getAction());
			return new BocloudResult(true, model, "create securityGroupRule success!");
		} else {
			logger.error("create securityGroupRule error ： " + object.getString(MESSAGE));
			return new BocloudResult(false, object.getString(MESSAGE));
		}
	}

	@Override
	public BocloudResult remove(String ruleId) {
		return null;
	}

	@Override
	public BocloudResult remove(SecurityGroupRuleModel model) {
		TreeMap<String, Object> params = new TreeMap<>();
		params.put("sgId", model.getGroupUuid());
		params.put("direction", model.getDirection());
		params.put("indexes.0", model.getSgrIndex());
		JSONObject object = this.connection(TceEnum.DFW, TceActionEnum.REMOVE_SECURITY_GROUP_RULE, params);
		if (this.verify(object)) {
			return new BocloudResult(true, "remove securityGroupRule success!");
		} else {
			logger.error("remove securityGroupRule error ： " + object.getString(MESSAGE));
			return new BocloudResult(false, object.getString(MESSAGE));
		}
	}

	public List<SecurityGroupRuleModel> convertModel(JSONObject object, String groupUuid) {
		List<SecurityGroupRuleModel> securityGroupRuleModels = new ArrayList<>();
		JSONArray ingressArray = object.getJSONArray("ingress");
		JSONArray egressArray = object.getJSONArray("egress");
		if (ingressArray.size() == 0 && egressArray.size() == 0) {
			return securityGroupRuleModels;
		}
		for (int i = 0; i < ingressArray.size(); i++) {
			JSONObject modelObject = ingressArray.getJSONObject(i);
			SecurityGroupRuleModel model = new SecurityGroupRuleModel();
			model.setGroupUuid(groupUuid);
			model.setSgrIndex(modelObject.getInteger("index"));
			model.setRemoteIpPrefix(modelObject.getString("cidrIp"));
			model.setRemark(modelObject.getString("desc"));
			model.setProtocol(modelObject.getString("ipProtocol").toUpperCase());
			if (!modelObject.getString("portRange").equalsIgnoreCase("ALL")) {
				String port = modelObject.getString("portRange");
				if (null != port && port.contains(",")) {
					// 目前不支持 不连续的端口
				} else if (null != port && port.contains("-")) {
					String[] split = port.split("-");
					model.setPortMin(Integer.parseInt(split[0]));
					model.setPortMax(Integer.parseInt(split[1]));
				} else {
					model.setPortMin(Integer.parseInt(port));
					model.setPortMax(Integer.parseInt(port));
				}
			}
			model.setDirection("ingress");
			model.setAction(modelObject.getString("action"));
			securityGroupRuleModels.add(model);
		}

		for (int i = 0; i < egressArray.size(); i++) {
			JSONObject modelObject = egressArray.getJSONObject(i);
			SecurityGroupRuleModel model = new SecurityGroupRuleModel();
			model.setGroupUuid(groupUuid);
			model.setSgrIndex(modelObject.getInteger("index"));
			model.setRemoteIpPrefix(modelObject.getString("cidrIp"));
			model.setRemark(modelObject.getString("desc"));
			model.setProtocol(modelObject.getString("ipProtocol").toUpperCase());
			if (!modelObject.getString("portRange").equalsIgnoreCase("ALL")) {
				String port = modelObject.getString("portRange");
				if (null != port && port.contains(",")) {
					// 目前不支持 不连续的端口
				} else if (null != port && port.contains("-")) {
					String[] split = port.split("-");
					model.setPortMin(Integer.parseInt(split[0]));
					model.setPortMax(Integer.parseInt(split[1]));
				} else {
					model.setPortMin(Integer.parseInt(port));
					model.setPortMax(Integer.parseInt(port));
				}
			}
			model.setDirection("egress");
			model.setAction(modelObject.getString("action"));
			securityGroupRuleModels.add(model);
		}
		return securityGroupRuleModels;
	}

}
