package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.List;
import java.util.TreeMap;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.TceActionEnum;
import com.bocloud.cmp.common.enums.TceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.TceRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.tceconvertor.SubnetConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: TceSubnetProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.tce")
public class TceSubnetProvider extends TceRestApiProvider implements SubnetProvider {

	private static Logger logger = LoggerFactory.getLogger(TceSubnetProvider.class);

	private BeanJsonConvertor<SubnetModel> convertor = new SubnetConvertor();

	public TceSubnetProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		Integer offset = 0;
		TreeMap<String, Object> params = new TreeMap<>();
		params.put("limit", LIMIT);
		List<SubnetModel> list = new ArrayList<>();
		JSONObject object = this.connection(TceEnum.VPC, TceActionEnum.LIST_SUBNET, params);
		if (this.verify(object)) {
			list.addAll(convertor.convertList(object.getJSONArray("data")));
			while (object.getInteger("totalCount") >= offset + LIMIT) {
				params.put("offset", (offset += LIMIT));
				object = this.connection(TceEnum.VPC, TceActionEnum.LIST_SUBNET, params);
				list.addAll(convertor.convertList(object.getJSONArray("data")));
			}
			return new BocloudResult(true, list, "list subnet success!");
		} else {
			logger.error("list subnet error ： " + object.getString(MESSAGE));
			return new BocloudResult(false, object.getString(MESSAGE));
		}
	}

	@Override
	public BocloudResult create(SubnetModel subnet) {
		TreeMap<String, Object> params = new TreeMap<>();
		params.put("vpcId", subnet.getVpcUuid());
		params.put("subnetSet." + subscript + ".subnetName", subnet.getName());
		params.put("subnetSet." + subscript + ".cidrBlock", subnet.getCidr());
		params.put("subnetSet." + subscript + ".zoneId", subnet.getZoneNum());
		JSONObject object = this.connection(TceEnum.VPC, TceActionEnum.CREATE_SUBNET, params);
		if (this.verify(object)) {
			SubnetModel convertModel = convertor.convertModel(object.getJSONArray("subnetSet").getJSONObject(0));
			convertModel.setRegionId(this.getRegionId());
			convertModel.setZone(subnet.getZone());
			return new BocloudResult(true, convertModel, "create subnet success!");
		} else {
			logger.error("create subnet error ： " + object.getString(MESSAGE));
			return new BocloudResult(false, object.getString(MESSAGE));
		}
	}

	@Override
	public BocloudResult modify(SubnetModel subnet) {
		return null;
	}

	@Override
	public BocloudResult remove(String subnetId) {
		return null;
	}

	@Override
	public BocloudResult detail(String subnetId) {
		TreeMap<String, Object> params = new TreeMap<>();
		params.put("limit", LIMIT);
		params.put("subnetId", subnetId);
		JSONObject object = this.connection(TceEnum.VPC, TceActionEnum.LIST_SUBNET, params);
		if (this.verify(object)) {
			JSONArray jsonArray = object.getJSONArray("data");
			return new BocloudResult(true, convertor.convertModel(jsonArray.getJSONObject(0)), "list subnet success!");
		} else {
			logger.error("list subnet error ： " + object.getString(MESSAGE));
			return new BocloudResult(false, object.getString(MESSAGE));
		}
	}

	@Override
	public BocloudResult remove(SubnetModel subnet) {
		TreeMap<String, Object> params = new TreeMap<>();
		params.put("vpcId", subnet.getVpcUuid());
		params.put("subnetId", subnet.getSubnetUuid());
		JSONObject object = this.connection(TceEnum.VPC, TceActionEnum.REMOVE_SUBNET, params);
		if (this.verify(object)) {
			return new BocloudResult(true, "remove subnet success!");
		} else {
			logger.error("remove subnet error ： " + object.getString(MESSAGE));
			return new BocloudResult(false, object.getString(MESSAGE));
		}
	}
}
