package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.List;
import java.util.TreeMap;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.TceActionEnum;
import com.bocloud.cmp.common.enums.TceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.TceRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.tceconvertor.VpcConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: TceVpcProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.tce")
public class TceVpcProvider extends TceRestApiProvider implements VpcProvider {

    private static Logger logger = LoggerFactory.getLogger(TceVpcProvider.class);

    private BeanJsonConvertor<VpcModel> convertor = new VpcConvertor();

    public TceVpcProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(VpcModel model) {
        TreeMap<String, Object> params = new TreeMap<>();
        params.put("vpcName", model.getName());
        params.put("cidrBlock", model.getCidr());
        JSONObject object = this.connection(TceEnum.VPC, TceActionEnum.CREATE_VPC, params);
        if (this.verify(object)) {
            model.setVpcId(object.getString("uniqVpcId"));
            model.setDefaultVpc(false);
            model.setStatus("ACTIVE");
            return new BocloudResult(true, model, "create vpc success!");
        } else {
            logger.error("create vpc error ： " + object.getString(MESSAGE));
            return new BocloudResult(false, object.getString(MESSAGE));
        }
    }

    @Override
    public BocloudResult remove(String vpcId) {
        TreeMap<String, Object> params = new TreeMap<>();
        params.put("vpcId", vpcId);
        JSONObject object = this.connection(TceEnum.VPC, TceActionEnum.REMOVE_VPC, params);
        if (this.verify(object)) {
            return new BocloudResult(true, "remove vpc success!");
        } else {
            logger.error("remove vpc error ： " + object.getString(MESSAGE));
            return new BocloudResult(false, object.getString(MESSAGE));
        }
    }

    @Override
    public BocloudResult list() {
        Integer offset = 0;
        TreeMap<String, Object> params = new TreeMap<>();
        params.put("limit", LIMIT);
        List<VpcModel> list = new ArrayList<>();
        JSONObject object = this.connection(TceEnum.VPC, TceActionEnum.LIST_VPC, params);
        if (this.verify(object)) {
            list.addAll(convertor.convertList(object.getJSONArray("data")));
            while (object.getInteger("totalCount") >= offset + LIMIT) {
                params.put("offset", (offset += LIMIT));
                object = this.connection(TceEnum.VPC, TceActionEnum.LIST_VPC, params);
                list.addAll(convertor.convertList(object.getJSONArray("data")));
            }
            return new BocloudResult(true, list, "list vpc success!");
        } else {
            logger.error("list vpc error ： " + object.getString(MESSAGE));
            return new BocloudResult(false, object.getString(MESSAGE));
        }
    }

    @Override
    public BocloudResult detail(String vpcId, String regionId) {
        try {
            TreeMap<String, Object> params = new TreeMap<>();
            params.put("vpcId", vpcId);
            JSONObject object = this.connection(TceEnum.VPC, TceActionEnum.LIST_VPC, params);
            List<VpcModel> vpcModels = new ArrayList<>();
            if (this.verify(object)) {
                vpcModels = convertor.convertList(object.getJSONArray("data"));
            }
            VpcModel vpcModel = vpcModels.get(0);
            vpcModel.setRegionId(regionId);
            return new BocloudResult(true, vpcModel, "detail vpc success!");
        } catch (Exception e) {
            logger.error("detail vpc error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult update(VpcModel model) {
        TreeMap<String, Object> params = new TreeMap<>();
        params.put("vpcId", model.getVpcId());
        params.put("vpcName", model.getName());
        JSONObject object = this.connection(TceEnum.VPC, TceActionEnum.MODIFY_VPC, params);
        if (this.verify(object)) {
            return this.detail(model.getVpcId(), model.getRegionId());
        } else {
            logger.error("modify vpc error ： " + object.getString(MESSAGE));
            return new BocloudResult(false, object.getString(MESSAGE));
        }
    }

}
