package com.bocloud.cmp.provider.security;

import java.util.Arrays;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.common.enums.TceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.TceProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tceconvertor.KeypairConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.cvm.v20170312.CvmClient;
import com.tencentcloudapi.cvm.v20170312.models.CreateKeyPairRequest;
import com.tencentcloudapi.cvm.v20170312.models.CreateKeyPairResponse;
import com.tencentcloudapi.cvm.v20170312.models.DeleteKeyPairsRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeKeyPairsRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeKeyPairsResponse;
import com.tencentcloudapi.cvm.v20170312.models.ImportKeyPairRequest;
import com.tencentcloudapi.cvm.v20170312.models.ImportKeyPairResponse;
import com.tencentcloudapi.cvm.v20170312.models.KeyPair;
import com.tencentcloudapi.cvm.v20170312.models.ModifyKeyPairAttributeRequest;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: TceKeypairProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.tce")
public class TceKeypairProvider extends TceProvider implements KeypairProvider {

	private static Logger logger = LoggerFactory.getLogger(TceKeypairProvider.class);

	private BeanConvertor<KeypairModel, KeyPair> convertor = new KeypairConvertor();

	public TceKeypairProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			DescribeKeyPairsResponse response = client.DescribeKeyPairs(new DescribeKeyPairsRequest());
			return new BocloudResult(true, convertor.convertList(Arrays.asList(response.getKeyPairSet())),
					"list keypair success!");
		} catch (Exception e) {
			logger.error("list keypair error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String keypairName) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			DescribeKeyPairsRequest request = new DescribeKeyPairsRequest();
			request.setKeyIds(new String[] { keypairName });
			DescribeKeyPairsResponse response = client.DescribeKeyPairs(request);
			KeyPair[] keyPairs = response.getKeyPairSet();
			KeyPair key = null;
			if (null != keyPairs && keyPairs.length > 0) {
				key = keyPairs[0];
			}
			return new BocloudResult(true, convertor.convertModel(key), "detail keypair success!");
		} catch (Exception e) {
			logger.error("detail keypair error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult create(String keypairName, String publicKey) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			if (!StringUtils.isEmpty(publicKey)) {
				ImportKeyPairRequest request = new ImportKeyPairRequest();
				request.setKeyName(keypairName);
				request.setPublicKey(publicKey);
				if (!StringUtils.isEmpty(this.getProjectId())) {
					request.setProjectId(Integer.valueOf(this.getProjectId()));
				}
				ImportKeyPairResponse response = client.ImportKeyPair(request);
				return this.detail(response.getKeyId());
			} else {
				CreateKeyPairRequest req = new CreateKeyPairRequest();
				req.setKeyName(keypairName);
				if (!StringUtils.isEmpty(this.getProjectId())) {
					req.setProjectId(Integer.valueOf(this.getProjectId()));
				}
				CreateKeyPairResponse resp = client.CreateKeyPair(req);
				return new BocloudResult(true, convertor.convertModel(resp.getKeyPair()), "create keypair success!");
			}
		} catch (Exception e) {
			logger.error("create keypair error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult delete(String keypairName) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			DeleteKeyPairsRequest request = new DeleteKeyPairsRequest();
			request.setKeyIds(new String[] { keypairName });
			client.DeleteKeyPairs(request);
			return new BocloudResult(true, "delete keypair success!");
		} catch (Exception e) {
			logger.error("delete keypair error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(KeypairModel model) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			ModifyKeyPairAttributeRequest request = new ModifyKeyPairAttributeRequest();
			request.setKeyName(model.getName());
			request.setKeyId(model.getKeyId());
			client.ModifyKeyPairAttribute(request);
			return new BocloudResult(true, "modify keypair success!");
		} catch (Exception e) {
			logger.error("modify keypair error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

}
