package com.bocloud.cmp.tceconvertor;

import java.util.ArrayList;
import java.util.List;

import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.tencentcloudapi.cvm.v20170312.models.Image;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: ImageConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
public class ImageConvertor implements BeanConvertor<ImageModel, Image> {

	@Override
	public List<ImageModel> convertList(List<? extends Image> objects) {
		List<ImageModel> imageModels = new ArrayList<ImageModel>();
		for (Image object : objects) {
			imageModels.add(convertModel(object));
		}
		return imageModels;
	}

	@Override
	public ImageModel convertModel(Image image) {
		ImageModel model = new ImageModel();
		model.setImageUuid(image.getImageId());
		model.setName(image.getImageName());
		if (image.getImageType().equalsIgnoreCase("PUBLIC_IMAGE")) {
			model.setImageType("PUBLIC");
		} else if (image.getImageType().equalsIgnoreCase("PRIVATE_IMAGE")) {
			model.setImageType("PRIVATE");
		} else if (image.getImageType().equalsIgnoreCase("SHARED_IMAGE")) {
			model.setImageType("SHARED");
		} else {
			model.setImageType(image.getImageType());
		}
		model.setOsName(image.getOsName());
		String platform = image.getPlatform();
		model.setOsType(platform);
		if (null != platform && "Windows".equals(platform)) {
			model.setOsCategory("Windows");
		} else {
			model.setOsCategory("Linux");
		}
		model.setRemark(image.getImageDescription());
		// 该代码因为腾讯环境数据异常，有条镜像大小为几千万G造成同步失败。
		if (image.getImageSize() > (10 * 1000)) {
			model.setSize(Double.valueOf(image.getImageSize() / (10 * 1000)));
			model.setMinDisk(Long.parseLong(image.getImageSize() / (10 * 1000) + ""));
		} else {
			model.setSize(Double.valueOf(image.getImageSize()));
			model.setMinDisk(Long.parseLong(image.getImageSize() + ""));
		}
		model.setArchitecture(image.getArchitecture());
		String imageState = image.getImageState();
		if ("NORMAL".equals(imageState)) {
			model.setStatus("ACTIVE");
		} else {
			model.setStatus(imageState == null ? null : imageState.toUpperCase());
		}
		return model;
	}
}
