package com.bocloud.cmp.tceconvertor;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.tencentcloudapi.cvm.v20170312.models.DataDisk;
import com.tencentcloudapi.cvm.v20170312.models.Instance;
import com.tencentcloudapi.cvm.v20170312.models.InternetAccessible;
import com.tencentcloudapi.cvm.v20170312.models.LoginSettings;
import com.tencentcloudapi.cvm.v20170312.models.VirtualPrivateCloud;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: ServerConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
public class ServerConvertor implements BeanConvertor<ServerModel, Instance> {

	@Override
	public List<ServerModel> convertList(List<? extends Instance> objects) {
		List<ServerModel> serverModels = new ArrayList<ServerModel>();
		for (Instance object : objects) {
			serverModels.add(convertModel(object));
		}
		return serverModels;
	}

	@Override
	public ServerModel convertModel(Instance instance) {
		ServerModel model = new ServerModel();
		model.setCpu(instance.getCPU().intValue());
		model.setMemory(BigDecimal.valueOf(instance.getMemory()));
		model.setFlavorUuid(instance.getInstanceType());
		model.setImageUuid(instance.getImageId());
		model.setInstanceId(instance.getInstanceId());
		String instanceName = instance.getInstanceName();
		if ("未命名".equals(instanceName)) {
			instanceName = instance.getInstanceId();
		}
		if (!StringUtils.isEmpty(instance.getCreatedTime())) {
			model.setGmtVmCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", instance.getCreatedTime(), "Z"));
		}
		model.setName(instanceName);
		model.setStatus(instance.getInstanceState());
		String[] securityGroupIds = instance.getSecurityGroupIds();
		if (null != securityGroupIds && securityGroupIds.length > 0) {
			model.setSecurityGroups(Arrays.asList(securityGroupIds));
		}
		model.setSystemDiskCategory(instance.getSystemDisk().getDiskType());
		model.setOsName(instance.getOsName());
		model.setZone(instance.getPlacement().getZone());
		String[] privateIpAddresses = instance.getPrivateIpAddresses();
		if (null != privateIpAddresses && privateIpAddresses.length > 0) {
			model.setPrivateIps(JSON.toJSONString(privateIpAddresses));
		}
		String[] publicIpAddresses = instance.getPublicIpAddresses();
		if (null != publicIpAddresses && publicIpAddresses.length > 0) {
			model.setPublicIps(JSON.toJSONString(publicIpAddresses));
		}
		model.setSystemDiskSize(instance.getSystemDisk().getDiskSize().intValue());
		model.setSystemDiskCategory(instance.getSystemDisk().getDiskType());

		model.setInstanceChargeType(instance.getInstanceChargeType());
		InternetAccessible internetAccessible = instance.getInternetAccessible();
		model.setInternetMaxBandwidthOut(internetAccessible.getInternetMaxBandwidthOut().intValue());
		model.setInternetChargeType(internetAccessible.getInternetChargeType());
		model.setIsUsePublicIp(internetAccessible.getPublicIpAssigned());

		VirtualPrivateCloud virtualPrivateCloud = instance.getVirtualPrivateCloud();
		model.setNetworkUuid(virtualPrivateCloud.getVpcId());
		model.setSubnetUuid(virtualPrivateCloud.getSubnetId());

		List<VolumeModel> diskDevices = new ArrayList<>();
		VolumeModel volumeModel;
		DataDisk[] dataDisks = instance.getDataDisks();
		if (null != dataDisks && dataDisks.length > 0) {
			for (DataDisk disk : dataDisks) {
				volumeModel = new VolumeModel();
				volumeModel.setVolumeUuid(disk.getDiskId());
				volumeModel.setType(disk.getDiskType());
				volumeModel.setSize(disk.getDiskSize().intValue());
				diskDevices.add(volumeModel);
			}
		}
		model.setDiskDevices(diskDevices);
		LoginSettings loginSettings = instance.getLoginSettings();
		if (null != loginSettings) {
			if (null != loginSettings.getKeyIds() && loginSettings.getKeyIds().length > 0) {
				model.setKeypairName(JSON.toJSONString(loginSettings.getKeyIds()));
			}
		}
		return model;
	}

}
