package com.bocloud.cmp.tceconvertor;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.tencentcloudapi.cbs.v20170312.models.Disk;
import com.tencentcloudapi.cbs.v20170312.models.Placement;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: VolumeConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
public class VolumeConvertor implements BeanConvertor<VolumeModel, Disk> {

	@Override
	public List<VolumeModel> convertList(List<? extends Disk> objects) {
		List<VolumeModel> volumeModels = new ArrayList<VolumeModel>();
		for (Disk object : objects) {
			volumeModels.add(convertModel(object));
		}
		return volumeModels;
	}

	@Override
	public VolumeModel convertModel(Disk disk) {
		VolumeModel model = new VolumeModel();
		model.setVolumeUuid(disk.getDiskId());
		String diskName = disk.getDiskName();
		if (null != diskName && "未命名".equals(diskName)) {
			diskName = disk.getDiskId();
		}
		model.setName(diskName);
		model.setSize(disk.getDiskSize().intValue());
		model.setType(disk.getDiskType());
		String diskState = disk.getDiskState();
		if ("ATTACHED".equalsIgnoreCase(diskState)) {
			model.setStatus("IN_USE");
		} else if ("UNATTACHED".equalsIgnoreCase(diskState)) {
			model.setStatus("AVAILABLE");
		} else {
			model.setStatus(diskState);
		}
		// •PREPAID：预付费，即包年包月
		// •POSTPAID_BY_HOUR：后付费，即按量计费。
		model.setDiskChargeType(disk.getDiskChargeType());
		if (!StringUtils.isEmpty(disk.getDeadlineTime())) {
			model.setDiskExpireTime(DateTools.stringToDateTime(disk.getDeadlineTime()));
		}
		model.setInstanceId(disk.getInstanceId());
		model.setDeleteWithInstance(disk.getDeleteWithInstance());
		model.setBootVolume(disk.getDeleteWithInstance());
		Placement placement = disk.getPlacement();
		model.setZone(placement.getZone());
		// •SYSTEM_DISK：系统盘
		// •DATA_DISK：数据盘。
		String diskUsage = disk.getDiskUsage();
		model.setUsage(disk.getDiskUsage());
		if (null != diskUsage && "SYSTEM_DISK".equalsIgnoreCase(diskUsage)) {
			model.setBootVolume(true);
		}
		if (!StringUtils.isEmpty(disk.getCreateTime())) {
			model.setCreateTime(DateTools.stringToDateTime(disk.getCreateTime()));
		}
		return model;
	}

}
